#include <unistd.h>
#include <syslog.h>
#include <signal.h>
#include <sys/wait.h>
#include <ctype.h>
#include <string.h>
#include <openssl/des.h>
#include <fcntl.h>

#define ZEND_API_PSA_RESTRICTED
#include "../ext/psakey/php_psakey.h"
#include "../ext/psakeyupdate/php_psakeyupdate.h"
#include "plesk.h"

#define DES_PACE 8
#define BUF_SIZE 16384

const char ALERT_MESS[] = "<html>\n"
"<head> <title>Key Error</title>\n"
"<body bgcolor='#ffffff'>\n"
"<script> <!-- \n"
"alert('The key that you are using is invalid for the product version you are running.\\n"
"To purchase a new key go to the Plesk Online Store at www.sw-soft.com or contact\\n"
"sales@sw-soft.com.');\n"
"//--> </script>\n"
"</body> </html>\n";

extern char *bsg_php_prefix;
extern int bsg_php_prefix_real_len;
int in = 0, out = 1;
des_cblock des_buf_local[DES_PACE];
des_cblock key_local[DES_PACE];
extern des_key_schedule schedule;

#define MAX_RASSH 256	// maximum decryption child processes
int rassh_pids[MAX_RASSH];	// decryption childs pids

int
is_crypted_body(int fd, size_t offset)
{
	char c;

	lseek(fd, offset, SEEK_SET);
	
	while (read(fd, &c, 1) > 0 && isspace(c));
	lseek(fd, offset, SEEK_SET);

	return (c != '<');
}

int
is_crypted(int fd, off_t* offset_p)
{
	const char head_begin[] = "#!";
	size_t head_begin_len = sizeof(head_begin) - 1;
	const char head_end[] = "/admin/bin/php\n";
	size_t head_end_len = sizeof(head_end) - 1;
	size_t buf_len;
	size_t root_d_len;
	
	autoprepend();
	root_d_len = strlen(PRODUCT_ROOT_D);

	buf_len = head_begin_len + root_d_len + head_end_len;
	
	{
		char buf[buf_len];
		size_t read_len;

		lseek(fd, 0, SEEK_SET);
		if ((read_len = read(fd, buf, buf_len)) == buf_len
			&& strncmp(buf, head_begin, head_begin_len) == 0
			&& strncmp(buf + head_begin_len, PRODUCT_ROOT_D, root_d_len) == 0
			&& strncmp(buf + head_begin_len + root_d_len, head_end, head_end_len) == 0)
		{
			*offset_p = buf_len;
			return is_crypted_body(fd, *offset_p);
		}
	}

	*offset_p = 0;
	return is_crypted_body(fd, *offset_p);
}

int
is_bad_key(int in_par, off_t offset)
{
	int numr, in = in_par;
	char buf[DES_PACE], mem_image[DES_PACE]; //sizes are equial
	int i = 0;
	struct key_prop_s* prop;

	#warning "this code is deprecated now (see psakeyupdate&psakey exts)"
	prop = find_prop("key_restrict", 0);
	if (prop->kp_name == NULL || prop->kp_vtype != kp_vtype_int || KP_INT(*prop) != 0) {
		return 1;
	}

	lseek(in, offset, SEEK_SET);

	numr = read(in, buf, DES_PACE);
	des_ecb_encrypt((des_cblock *)buf, des_buf_local, schedule, DES_DECRYPT);
	memcpy(mem_image, des_buf_local, numr);

	while (((i<numr) && isspace(mem_image[i]))) {
		i++;
	}

	lseek(in, offset, SEEK_SET);

	return (mem_image[i] != '<');
}

void
crypt_file(int in_par, int out_par)
{
	int numr, mem_image_offset = 0;
	int p, numfile = 0;
	int in = in_par, out = out_par;
	char file_buf[BUF_SIZE], mem_image[BUF_SIZE]; //sizes are equial
	char buf[DES_PACE];

	while ((numfile = read(in, file_buf, BUF_SIZE))) {
		for (p = 0; p < numfile; p += DES_PACE ) {
			numr = (numfile - p)%DES_PACE;
			if (!numr) numr = DES_PACE;
			des_ecb_encrypt((des_cblock *)(file_buf + p), des_buf_local, schedule, DES_DECRYPT);
			memcpy(mem_image + mem_image_offset, des_buf_local, numr);
			mem_image_offset += numr;
		}
		write(out, mem_image, mem_image_offset);
		mem_image_offset = 0;
	}
}

void
no_crypt_file(int in_par, int out_par)
{
	int numr, in = in_par, out = out_par;
	char buf[BUF_SIZE];

	while ((numr = read(in, buf, sizeof(buf)))) {
			write(out, buf, numr);
	}
}

void
write_prefix_simple(int out)
{
	write(out, bsg_php_prefix, bsg_php_prefix_real_len);
}

void
new_rassh(int in_fd, int out_fd, int non_inc)
{
	int	numr;
	off_t offset;

	in = in_fd;
	out = out_fd;

	/* put the limits in the stream */
	write_prefix_simple(out);


	if (is_crypted(in_fd, &offset)) {
		if (is_bad_key(in, offset)) {
			if (non_inc) write(out, ALERT_MESS, strlen(ALERT_MESS));
			exit(1);
		}
		crypt_file(in, out);
	} else {
		no_crypt_file(in, out);
	}

	close(in);
	close(out);
}

#if !defined(__sun__)
void
bsg_reapchild(int signo)
{
	register int i = MAX_RASSH;
	int	pid;

	while (i--) {
		if (0 == rassh_pids[i])
			continue;
		pid = waitpid(rassh_pids[i], NULL, WNOHANG);
		if (pid == rassh_pids[i] || ((-1 == pid) && (ECHILD == errno)))
			rassh_pids[i] = 0;				
	}
}
#endif


#if defined(PLESK_DEBUG)
psa_debug(const char *s1, const char *s2, const char *s3)
{
	FILE *psa_fh = NULL;
	char *debug_filename = "/tmp/psa_debug.log";

	mk_lock(debug_filename, 1);

	psa_fh = fopen(debug_filename, "a");

	if (psa_fh) {
		fprintf(psa_fh, "%s %s %s\n", s1, s2, s3);
		fclose(psa_fh);
	}

	rm_lock(debug_filename, 1);

}
#endif

FILE *
rassh(FILE * fp_in)
{
	int pid, pipe_fd[2];
	register int i = -1;

	if (pipe(pipe_fd) < 0) {
		openlog("httpsd", LOG_PID | LOG_NDELAY, LOG_DAEMON);
		syslog(LOG_DEBUG, "can't open pipe");
		closelog();
		return NULL;
	}

	/*
	 * apache redefines signal() to ap_signal()
	 * the following #undef should be removed and this function
	 * should be moved to the top of file (before apache includes)
	 * if there will be other code in this file which relies on ap_signal
	 * (there is now such code at this time)
	 */
#undef signal
#if !defined(__sun__)
	signal(SIGCHLD, bsg_reapchild);
#else
	signal(SIGCHLD, SIG_IGN);
#endif

	pid = fork();
	if (pid == -1) {
		openlog("httpsd", LOG_PID | LOG_NDELAY, LOG_DAEMON);
		syslog(LOG_ERR, "fork failed");
		closelog();
		return NULL;
	}

	if (pid == 0) {
		/* child */
		close(pipe_fd[0]);
		new_rassh(fileno(fp_in), pipe_fd[1], apache_main_process > 0);
		close(pipe_fd[1]);
		fclose(fp_in);
		exit(0);
	}

	// save child pid to the pids list to call waitpid() in bsg_reapchild only to
	// our decryption child processes, and not for others, like PHP's mail(), exec() etc.
	// see http://bugs/show_bug.cgi?id=22716

	// find the empty cell
	while ((i < MAX_RASSH) && rassh_pids[++i])
			;
	// save pid
	// we suppose that MAX_RASSH decryption childs working for each apache process at the same
	// time will be enaugh, in other case we just loose child pid, and child becomes zombie.
	if (i < MAX_RASSH)
		rassh_pids[i] = pid;

	/* parent */
	close(pipe_fd[1]);
	fclose(fp_in);
	return fdopen(pipe_fd[0], "r");

}
